window.Gleam.initSearch([{"doc":"paint","title":"paint","content":"# Paint\n**Make drawings, animations, and games with Gleam!**\n\n[![Package Version](https://img.shields.io/hexpm/v/paint)](https://hex.pm/packages/paint)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/paint/)\n\nPaint is a tiny embedded domain specific language (inspired by [Gloss](https://hackage.haskell.org/package/gloss)).\nMake pictures out of basic shapes then style, transform, and combine them using the provided functions.\n\n![Frame 3(2)](https://github.com/user-attachments/assets/a8b83b58-990a-432a-9034-deebc4d210a6)\n\n\n```gleam\nimport paint\n\nfn main() {\n  let my_picture = paint.combine([\n    paint.circle(50.0),\n    paint.circle(30.0) |&gt; paint.fill(paint.colour_rgb(0, 200, 200)),\n    paint.rectangle(100.0, 50.0) |&gt; paint.rotate(angle_deg(30.0)),\n    paint.text(&quot;Hello world&quot;, 20) |&gt; paint.translate_y(-65.0),\n  ])\n\n  paint.display_on_canvas(fn(_canvas_config) { my_picture }, &quot;#canvas_id&quot;)\n}\n```\n\n**Want to learn more? Read the [docs](https://hexdocs.pm/paint) or browse the [visual examples](https://adelhult.github.io/paint/).**\n\n&gt; [!NOTE]\n&gt; Currently, there is only a HTML canvas backend for displaying the pictures.\n&gt; Hopefully, I will have time to add other backends in the future (such as SVG). Feel free to contribute!\n\n## Logo\nLucy is borrowed from the [Gleam branding page](https://gleam.run/branding/) and the brush is made by [Delapouite (game icons)](https://game-icons.net/1x1/delapouite/paint-brush.html).\n\n## Changelog\nThe API is considered unstable until a `1.*.*` release is published.\nChanges between versions can be found in the file `CHANGELOG.md`.\n\n## TODOs\n- [ ] I&#39;m not super happy with the API for the transformations. (Especially that scale affects the stroke width).\n      Would be fun to investigate an API more similar to [Haskell diagrams](https://hackage.haskell.org/package/diagrams) and [Diagrammer](https://www.youtube.com/watch?v=gT9Xu-ctNqI).\n- [x] Replace the color functions with [gleam-community/colour](https://hexdocs.pm/gleam_community_colour/)\n- [ ] Support (bitmap) images\n- [ ] Split the library into multiple files (would make integration with other libraries like Lustre more convenient).\n- [x] Allowing arbitrary css selectors instead of requiring ids in `interact_...` and `display_...`\n- [ ] Improve input handling for `interact_on_canvas`\n- [ ] Add another backend?\n","url":"index.html"},{"doc":"paint","title":"Angle","content":"pub opaque type Angle\n An angle in clock-wise direction.\n See: `angle_rad` and `angle_deg`.\n\nSynonyms:\npaint.Angle\npaint Angle","url":"paint.html#Angle"},{"doc":"paint","title":"CanvasConfig","content":"pub type CanvasConfig {\n  CanvasConfig(width: Float, height: Float)\n}\n The configuration of the &quot;canvas&quot;\nCanvasConfig(width: Float, height: Float)\n\n\nSynonyms:\npaint.CanvasConfig\npaint CanvasConfig","url":"paint.html#CanvasConfig"},{"doc":"paint","title":"Colour","content":"pub type Colour =\n  colour.Colour\n A rexport of the Colour type from [gleam_community/colour](https://hexdocs.pm/gleam_community_colour/).\n Paint also includes the functions `colour_hex` and `colour_rgb` to\n easily construct Colours, but feel free to import the `gleam_community/colour` module\n and use the many utility that are provided from there.\n\nSynonyms:\npaint.Colour\npaint Colour","url":"paint.html#Colour"},{"doc":"paint","title":"Event","content":"pub type Event {\n  Tick(Float)\n  KeyDown(Key)\n  KeyUp(Key)\n}\n A list of events\nTick(Float)\n Triggered before drawing. Contains the number of milliseconds elapsed.\n\nKeyDown(Key)\n Triggered when a key is pressed\n\nKeyUp(Key)\n Triggered when a key is released\n\nSynonyms:\npaint.Event\npaint Event","url":"paint.html#Event"},{"doc":"paint","title":"Key","content":"pub type Key {\n  LeftArrow\n  RightArrow\n  UpArrow\n  DownArrow\n  Space\n}\n\nLeftArrow\n\n\nRightArrow\n\n\nUpArrow\n\n\nDownArrow\n\n\nSpace\n\n\nSynonyms:\npaint.Key\npaint Key","url":"paint.html#Key"},{"doc":"paint","title":"Picture","content":"pub opaque type Picture\n A 2D picture\n\nSynonyms:\npaint.Picture\npaint Picture","url":"paint.html#Picture"},{"doc":"paint","title":"StrokeProperties","content":"pub type StrokeProperties {\n  NoStroke\n  SolidStroke(Colour, Float)\n}\n Options for strokes. Either no stroke or\n a stroke with some given colour and line width.\nNoStroke\n\n\nSolidStroke(Colour, Float)\n\n\nSynonyms:\npaint.StrokeProperties\npaint StrokeProperties","url":"paint.html#StrokeProperties"},{"doc":"paint","title":"Vec2","content":"pub type Vec2 =\n  #(Float, Float)\n\n\nSynonyms:\npaint.Vec2\npaint Vec2","url":"paint.html#Vec2"},{"doc":"paint","title":"angle_deg","content":"pub fn angle_deg(degrees: Float) -&gt; Angle\n Create an angle expressed in degrees\nSynonyms:\npaint.angle_deg\npaint angle_deg","url":"paint.html#angle_deg"},{"doc":"paint","title":"angle_rad","content":"pub fn angle_rad(radians: Float) -&gt; Angle\n Create an angle expressed in radians\nSynonyms:\npaint.angle_rad\npaint angle_rad","url":"paint.html#angle_rad"},{"doc":"paint","title":"arc","content":"pub fn arc(radius: Float, start: Angle, end: Angle) -&gt; Picture\n An arc with some radius going from some\n starting angle to some other angle in clock-wise direction\nSynonyms:\npaint.arc\npaint arc","url":"paint.html#arc"},{"doc":"paint","title":"blank","content":"pub fn blank() -&gt; Picture\n A blank image\nSynonyms:\npaint.blank\npaint blank","url":"paint.html#blank"},{"doc":"paint","title":"center","content":"pub fn center(picture: Picture) -&gt; fn(CanvasConfig) -&gt; Picture\n Utility to set the origin in the center of the canvas\nSynonyms:\npaint.center\npaint center","url":"paint.html#center"},{"doc":"paint","title":"circle","content":"pub fn circle(radius: Float) -&gt; Picture\n A circle with some given radius\nSynonyms:\npaint.circle\npaint circle","url":"paint.html#circle"},{"doc":"paint","title":"colour_hex","content":"pub fn colour_hex(string: String) -&gt; Colour\n A utility around [colour.from_rgb_hex_string](https://hexdocs.pm/gleam_community_colour/gleam_community/colour.html#from_rgb_hex_string)\n (from `gleam_community/colour`) that **panics** on an invalid hex code.\nSynonyms:\npaint.colour_hex\npaint colour_hex","url":"paint.html#colour_hex"},{"doc":"paint","title":"colour_rgb","content":"pub fn colour_rgb(red: Int, green: Int, blue: Int) -&gt; Colour\n A utility around [colour.from_rgb255](https://hexdocs.pm/gleam_community_colour/gleam_community/colour.html#from_rgb255)\n (from `gleam_community/colour`) that **panics** if the values are outside of the allowed range.\nSynonyms:\npaint.colour_rgb\npaint colour_rgb","url":"paint.html#colour_rgb"},{"doc":"paint","title":"combine","content":"pub fn combine(pictures: List(Picture)) -&gt; Picture\n Combine multiple pictures into one\nSynonyms:\npaint.combine\npaint combine","url":"paint.html#combine"},{"doc":"paint","title":"concat","content":"pub fn concat(\n  picture: Picture,\n  another_picture: Picture,\n) -&gt; Picture\n Concatenate two pictures\nSynonyms:\npaint.concat\npaint concat","url":"paint.html#concat"},{"doc":"paint","title":"define_web_component","content":"pub fn define_web_component() -&gt; Nil\n As an alternative to `display_on_canvas` you can use this web components API to\n display your pictures. This may be especially convenient when using a front-end\n framework such as Lustre.\n\n After calling this function you will be able to use the element `paint-canvas`:\n html\n &lt;paint-canvas&gt;&lt;/paint-canvas&gt;\n &lt;script&gt;\n   const myCanvas = document.querySelector(&quot;paint-canvas&quot;);\n   // When the `picture` property is set to a `Picture` object that picture\n   // will be displayed on the canvas.\n   myCanvas.picture = ...;\n &lt;/script&gt;\n \n\n A more detailed example for using this API together with Luster can be found in [this GitHub Gist](https://gist.github.com/adelhult/03c5916df891a06bec706e6f0842cd91).\nSynonyms:\npaint.define_web_component\npaint define_web_component","url":"paint.html#define_web_component"},{"doc":"paint","title":"display_on_canvas","content":"pub fn display_on_canvas(\n  init: fn(CanvasConfig) -&gt; Picture,\n  selector: String,\n) -&gt; Nil\n Display a picture on a HTML canvas element\n (specified by some [CSS Selector](https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_selectors)).\n\n Note: this function may only be called once the page has loaded and the\n document objects is available.\nSynonyms:\npaint.display_on_canvas\npaint display_on_canvas","url":"paint.html#display_on_canvas"},{"doc":"paint","title":"fill","content":"pub fn fill(picture: Picture, colour: Colour) -&gt; Picture\n Fill a picture with some given colour, see `Colour`.\nSynonyms:\npaint.fill\npaint fill","url":"paint.html#fill"},{"doc":"paint","title":"interact_on_canvas","content":"pub fn interact_on_canvas(\n  init: fn(CanvasConfig) -&gt; a,\n  update: fn(a, Event) -&gt; a,\n  view: fn(a) -&gt; Picture,\n  selector: String,\n) -&gt; Nil\n Make animations and games and display them on the given HTML canvas\n (specified by some [CSS Selector](https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_selectors)).\n\n Follows the same architecture as Elm/Lustre.\n The type variable &quot;state&quot; can be anything\n you want. If you are only making a stateless animation, use `Nil`.\n Note: this function may only be called once the page has loaded and the\n document and window objects are available.\nSynonyms:\npaint.interact_on_canvas\npaint interact_on_canvas","url":"paint.html#interact_on_canvas"},{"doc":"paint","title":"just","content":"pub fn just(picture: Picture) -&gt; fn(a) -&gt; Picture\n Utility function that is useful for cases where you\n are no interested in the canvas configuration. For example,\n \n display_on_canvas(just(circle(30.0)), &quot;#my_canvas&quot;)\n // instead of...\n display_on_canvas(fn(_config) { circle(30.0) }, &quot;#my_canvas&quot;)\n \nSynonyms:\npaint.just\npaint just","url":"paint.html#just"},{"doc":"paint","title":"lines","content":"pub fn lines(points: List(#(Float, Float))) -&gt; Picture\n Lines (same as a polygon but not a closed shape)\nSynonyms:\npaint.lines\npaint lines","url":"paint.html#lines"},{"doc":"paint","title":"polygon","content":"pub fn polygon(points: List(#(Float, Float))) -&gt; Picture\n A polygon consisting of a list of 2d points\nSynonyms:\npaint.polygon\npaint polygon","url":"paint.html#polygon"},{"doc":"paint","title":"rectangle","content":"pub fn rectangle(width: Float, height: Float) -&gt; Picture\n A rectangle with some given width and height\nSynonyms:\npaint.rectangle\npaint rectangle","url":"paint.html#rectangle"},{"doc":"paint","title":"rotate","content":"pub fn rotate(picture: Picture, angle: Angle) -&gt; Picture\n Rotate the picture in a clock-wise direction\nSynonyms:\npaint.rotate\npaint rotate","url":"paint.html#rotate"},{"doc":"paint","title":"scale_uniform","content":"pub fn scale_uniform(picture: Picture, factor: Float) -&gt; Picture\n Scale the picture uniformly in horizontal and vertical direction\nSynonyms:\npaint.scale_uniform\npaint scale_uniform","url":"paint.html#scale_uniform"},{"doc":"paint","title":"scale_x","content":"pub fn scale_x(picture: Picture, factor: Float) -&gt; Picture\n Scale the picture in the horizontal direction\nSynonyms:\npaint.scale_x\npaint scale_x","url":"paint.html#scale_x"},{"doc":"paint","title":"scale_y","content":"pub fn scale_y(picture: Picture, factor: Float) -&gt; Picture\n Scale the picture in the vertical direction\nSynonyms:\npaint.scale_y\npaint scale_y","url":"paint.html#scale_y"},{"doc":"paint","title":"square","content":"pub fn square(length: Float) -&gt; Picture\n A square\nSynonyms:\npaint.square\npaint square","url":"paint.html#square"},{"doc":"paint","title":"stroke","content":"pub fn stroke(\n  picture: Picture,\n  stroke_properties: StrokeProperties,\n) -&gt; Picture\n Set properties for the stroke. See `StrokeProperties`\nSynonyms:\npaint.stroke\npaint stroke","url":"paint.html#stroke"},{"doc":"paint","title":"text","content":"pub fn text(text: String, font_size_px: Int) -&gt; Picture\n Text with some given font size\nSynonyms:\npaint.text\npaint text","url":"paint.html#text"},{"doc":"paint","title":"translate_x","content":"pub fn translate_x(picture: Picture, x: Float) -&gt; Picture\n Translate a picture in the horizontal direction\nSynonyms:\npaint.translate_x\npaint translate_x","url":"paint.html#translate_x"},{"doc":"paint","title":"translate_xy","content":"pub fn translate_xy(\n  picture: Picture,\n  x: Float,\n  y: Float,\n) -&gt; Picture\n Translate a picture in horizontal and vertical direction\nSynonyms:\npaint.translate_xy\npaint translate_xy","url":"paint.html#translate_xy"},{"doc":"paint","title":"translate_y","content":"pub fn translate_y(picture: Picture, y: Float) -&gt; Picture\n Translate a picture in the vertical direction\nSynonyms:\npaint.translate_y\npaint translate_y","url":"paint.html#translate_y"},{"doc":"paint","title":"paint","content":"","url":"paint.html"}]);